/*
 * Copyright (c) 2015 Tricoire Sebastien 3dsman@free.fr
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 * claim that you wrote the original software. If you use this software
 * in a product, an acknowledgment in the product documentation would be
 * appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 *
 */

#ifndef OE_EDITDISPLAY_H
#define OE_EDITDISPLAY_H

#include "OE_utils.h"
#include <stdint.h>
#include <vector>
#include "OE_document.h"
#include "OE_display.h"
#include <GLFW/glfw3.h>
#include "curves/OE_joincurve.h"
#include "curves/OE_subcurve.h"

class OE_editDisplay : public OE_display
{
public:
	
		
        /** Default constructor */
        OE_editDisplay();
		/** Default destructor */
        virtual ~OE_editDisplay();
		
		virtual void showAll();
		
		virtual void scroll(double xoffset, double yoffset);
		
		//void drawframe();
        bool select(std::list<OE_curve*> selectedCurves, std::list<OE_stitchs*> selectedStitches);

		bool draw();
        bool draw(bool select);
		//bool forceRefresh = true;

    protected:
    private:
		
		bool changeDpi = true;
		
        /** \brief is displaying controls.	 */
        bool controls = false;
		
		OE_curve* selectedCurve = nullptr;
		OE_stitchs* selectedStitch = nullptr;

		/** \brief color of the curve	 */
        static  unsigned char lineColor[4] ;

		/** \brief color of the handle lines	 */
        static unsigned char controlLineColor[4] ;
		
		/** \brief color of the handle points	 */
        static unsigned char controlPointColor[4] ;
		
		/** \brief color of the curve start and end points	 */
        static unsigned char controlStartPointColor[4] ;
		
		/** \brief color of the curve	 */
        static  unsigned char stitchLineColor[4] ;
		
		/** \brief color of the handle points	 */
        static unsigned char stitchPointColor[4] ;
		
		/** \brief color of the curve start and end points	 */
        static unsigned char stitchStartPointColor[4] ;
		
		bool refresh();
		
		void drawTriangle(vector_2d point1, vector_2d point2, float size);
		
		bool drawCurve(OE_curve * curve);
		bool drawStitch(OE_stitchs * stitch);
        bool drawCurveControl(OE_curve * curve, bool select = false);
        bool drawStitchControl(OE_stitchs * stitch, bool select = false);
        bool drawJoincurveControl(OE_joincurve * curve, bool select);
		
};

#endif // OE_EDITDISPLAY_H
