/*
 * Copyright (c) 2015 Tricoire Sebastien 3dsman@free.fr
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 * claim that you wrote the original software. If you use this software
 * in a product, an acknowledgment in the product documentation would be
 * appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 *
 */

#include "OE_commandDisplay.h"
#include "OE_utils.h"

#include "OE_controller.h"


#include <iostream>

#include <GL/gl.h>
#include <cstdlib>
#include <math.h>
#include <cstdio>
#include <cstring>
#include <algorithm>

unsigned char OE_commandDisplay::lineColor[] = {0,160,192,255};
unsigned char OE_commandDisplay::commandColor[] = {100,60,92,255};
unsigned char OE_commandDisplay::startPointColor[] = {100,60,92,255};
unsigned char OE_commandDisplay::curPointColor[] = {200,50,50,255};
 

OE_commandDisplay::OE_commandDisplay()
{
}

OE_commandDisplay::OE_commandDisplay(OE_document* document,OE_controller* controller)
{
}

OE_commandDisplay::~OE_commandDisplay()
{
}

void OE_commandDisplay::key(int key, int scancode, int action, int mods)
{
	if((action == GLFW_RELEASE)&&(key == GLFW_KEY_SPACE))
		if(controller)
        {
            //controller->sendInstPoint();
            run = !run;
		}
};



/** \brief draw the document on screen
 *
 * \return true if all is ok
 *
 */

bool OE_commandDisplay::draw()
{
	if (document)
	{
        glPushMatrix();
		vector_2d zero = document->getZeroPoint();
		float size = 1.0f/document->getPulseByMm();
		glTranslatef(zero.x,zero.y,0);
		glScalef(size,size,size);
		vector_2d hoopSize = document->getHoopSize()/size/2;
		
		//draw hoop
		glLineWidth(4.0);
		glBegin(GL_LINE_STRIP);
			glColor4d(0,0,0,1);
			glVertex2f(-hoopSize.x,-hoopSize.y);
			glVertex2f(-hoopSize.x,hoopSize.y);
			
			glVertex2f(hoopSize.x,hoopSize.y);
			glVertex2f(hoopSize.x,-hoopSize.y);
			
			glVertex2f(-hoopSize.x,-hoopSize.y);
		glEnd();
		
		drawInstPoints();
        if (run) controller->sendInstPoint();

        glPopMatrix();
	}
	return true;
}

bool OE_commandDisplay::drawInstPoints()
{
	
		unsigned i;
		
		// Points
		glPointSize(3.0f);
		glColor4ubv(lineColor);
		glBegin(GL_POINTS);

		for (i = 1; i < document->instPoints.size(); i++) {
			glVertex2f(document->instPoints.at(i).x,document->instPoints.at(i).y);
		}
		glEnd();

		
		//draw lines
		
		unsigned pt = 0;
		glLineWidth(1.5);
		glBegin(GL_LINE_STRIP);
		
		for (i = 0; i < document->instCommand.size(); i++) {
			
			
			
			for (; pt < document->instCommand.at(i)->getIdPoint(); pt++)
				glVertex2f(document->instPoints.at(pt).x,document->instPoints.at(pt).y);
			
			OE_waitcolor* color = dynamic_cast<OE_waitcolor*> (document->instCommand.at(i));
			if (color)
			{
				glColor4ubv(color->getThreadColor());
			}
			OE_start* start = dynamic_cast<OE_start*> (document->instCommand.at(i));
			if (start)
			{
				glColor4ubv(start->getThreadColor());
			}
		}
		
		for (; pt < document->instPoints.size(); pt++)
			glVertex2f(document->instPoints.at(pt).x,document->instPoints.at(pt).y);
			
		glEnd();
			
		// start point

		glColor4ubv(startPointColor);
		glPointSize(5.0f);
		glBegin(GL_POINTS);
		glVertex2f(document->instPoints.at(0).x,document->instPoints.at(0).y);
		glEnd();
		
		if (document->curPoint < document->instPoints.size())
		{
			glColor4ubv(curPointColor);
			glPointSize(5.0f);
			glBegin(GL_POINTS);
			glVertex2f(document->instPoints.at(document->curPoint).x,document->instPoints.at(document->curPoint).y);
			glEnd();
		}

	return true;
}
