/*
 * Copyright (c) 2015 Tricoire Sebastien 3dsman@free.fr
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 * claim that you wrote the original software. If you use this software
 * in a product, an acknowledgment in the product documentation would be
 * appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 *
 */

#include "stitchs/OE_linestitch.h"
#include "curves/OE_pointcurve.h"
#include "curves/OE_joincurve.h"
#include "curves/OE_subcurve.h"
#include <iostream>

OE_linestitch::OE_linestitch()
{
}

OE_linestitch::OE_linestitch(OE_thread* thread, float len, float width)
{
    curve = new OE_joincurve();
	curve->addDependency(this);
	this->len = len;
	this->width = width;
    this->thread = thread;
    setNeedRefresh();
}

OE_linestitch::OE_linestitch(OE_thread* thread, float len, float width, unsigned motif, float offset)
{
    curve = new OE_joincurve();
	curve->addDependency(this);
	this->len = len;
	this->width = width;
	this->stitchType=motif;
	this->offset=offset;
	this->thread = thread;
    setNeedRefresh();
}


OE_linestitch::~OE_linestitch()
{
    std::list<OE_subcurve*>::iterator i = curve->curves.begin();
     while (i != curve->curves.end())
     {
         (*i)->removeDependency(curve);
         delete *i;
         i++;
     }
    if (curve) delete curve;
}
/*
void  OE_linestitch::setCurve(OE_curve* curve)
{
	if (this->curve)this->curve->removeDependency(this);
	this->curve = curve;
	curve->addDependency(this);
	setNeedRefresh();
}
*/
void  OE_linestitch::setLen(float len){this->len = len; setNeedRefresh();}
void  OE_linestitch::setWidth(float width){this->width = width; setNeedRefresh();}
void  OE_linestitch::setMotif(unsigned stitch){stitchType=stitch; setNeedRefresh();}
void  OE_linestitch::setOffset(float offset){this->offset=offset; setNeedRefresh();}
void  OE_linestitch::setReverse(bool reverse){this->reverse=reverse; setNeedRefresh();}

OE_joincurve* OE_linestitch::getJoincurve(){return curve;}
float OE_linestitch::getLen(){return len;}
float OE_linestitch::getWidth(){return width;}
unsigned OE_linestitch::getMotif(){return stitchType;}
float OE_linestitch::getOffset(){return offset;}
bool OE_linestitch::getReverse(){return reverse;}

bool OE_linestitch::initMotifs()
{
	motifs.push_back(motif_line({vector_2d(0,0)},false)); //droit
	//motifs.push_back(motif_line({vector_2d(0.35,1),vector_2d(0.15,-1),vector_2d(0.85,-1),vector_2d(0.65,1)},false)); //satin
	motifs.push_back(motif_line({vector_2d(0.5,0),vector_2d(0,-0.5),vector_2d(1,-0.5),vector_2d(0,0.5),vector_2d(1,0.5),vector_2d(0.5,0)},false)); //satin
	motifs.push_back(motif_line({vector_2d(0,1),vector_2d(0.5,-1)},false)); //satin
	motifs.push_back(motif_line({vector_2d(0,0),vector_2d(0,1)},false)); //eclair
	motifs.push_back(motif_line({vector_2d(0,0),vector_2d(0,1),vector_2d(0,0)},false)); //pics
	motifs.push_back(motif_line({vector_2d(0,0),vector_2d(0.5,0),vector_2d(0.5,0.6),vector_2d(0.3,0.8),vector_2d(0.5,1.0),vector_2d(0.7,0.8),vector_2d(0.5,0.6),vector_2d(0.5,0)},false)); //fleur
	return true;
}


bool OE_linestitch::check(){return (curve&&curve->check()&&stitchType<motifs.size()&&len);}
//OE_curve::refresh(float dpi)
void OE_linestitch::refresh()
{

}

void OE_linestitch::refresh(float dpi)
{
    std::vector<vector_2dt> curvePts;
	if (check()&&needRefresh)
	{
        curve->refresh( dpi);
		if (curve->getNpts()>3)
		{
			
			curvePts = curve->discretizeRegular(len);

			if (curvePts.size()<2) return;
			
			//calcul des normales
			std::vector<vector_2d> 	normalPts;
			vector_2d tmpNormal;
			vector_2d normal;
			for(unsigned i=0; i<curvePts.size()-1; i++)
			{
                normal = curvePts.at(i+1).v-curvePts.at(i).v;
				normal.turnRight();
				if (i==0)
					tmpNormal = normal;
				tmpNormal =tmpNormal + normal;
				tmpNormal.normalize();
				normalPts.push_back( tmpNormal*width);
			
				tmpNormal = normal;
			}
			tmpNormal.normalize();
			normalPts.push_back( tmpNormal*width);
			
			
			motif_line motifeu = motifs.at(stitchType);
			//motif_line motifeu = motifs.at(3);
			//sortie des points
			std::vector<vector_2d> 	outPts;
			vector_2d pt,dir, vec1,vec2,tmp1,tmp2;
			for(unsigned i=0; i<curvePts.size()-1; i++)
			{
                dir = curvePts.at(i+1).v-curvePts.at(i).v;
				vec1 = normalPts.at(i);
				vec2 = normalPts.at(i+1);
				for(unsigned p = 0;p<motifeu.pts.size();p++)
				{
					pt = motifeu.pts.at(p);
					pt.y += offset;
					tmp1 = (vec1*pt.y)*(1-pt.x)+(vec2*pt.y)*pt.x;
					tmp2 = dir*pt.x;
                    outPts.push_back(curvePts.at(i).v+tmp1+tmp2);
				
				}
			}
			
			pts.clear();
		
			if(maxLen>0)
				pts = stitchMaxLen(outPts, maxLen);
			else
				pts = outPts;
		}
	}
}

void OE_linestitch::delDependency(OE_base * object)
{
	if (curve == object) curve = 0;
}
