/*
 * Copyright (c) 2015 Tricoire Sebastien 3dsman@free.fr
 *
 * This software is provided 'as-is', without any express or implied
 * warranty.  In no event will the authors be held liable for any damages
 * arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose,
 * including commercial applications, and to alter it and redistribute it
 * freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 * claim that you wrote the original software. If you use this software
 * in a product, an acknowledgment in the product documentation would be
 * appreciated but is not required.
 * 2. Altered source versions must be plainly marked as such, and must not be
 * misrepresented as being the original software.
 * 3. This notice may not be removed or altered from any source distribution.
 *
 */

#include "stitchs/OE_linkstitch.h"
#include "curves/OE_curve.h"
#include <iostream>

float OE_linkstitch::defaultLinkMaxlen = 5.0f;
/*
OE_linkstitch::OE_linkstitch()
{
}
*/
OE_linkstitch::OE_linkstitch(OE_stitchs* stitchStart, OE_stitchs* stitchEnd)
{
	this->stitchStart = stitchStart;
	stitchStart->addDependency(this);
	this->stitchEnd = stitchEnd;
	stitchEnd->addDependency(this);
	maxLen = defaultLinkMaxlen;
	thread = stitchStart->getThread();

    setNeedRefresh();
    //refresh();
}


OE_linkstitch::~OE_linkstitch()
{
    stitchStart->removeDependency(this);
    stitchEnd->removeDependency(this);
}

void  OE_linkstitch::setStitchStart(OE_stitchs* stitch)
{
	if (this->stitchStart)this->stitchStart->removeDependency(this);
	this->stitchStart = stitch;
	stitch->addDependency(this);
	setNeedRefresh();
}

void  OE_linkstitch::setStitchEnd(OE_stitchs* stitch)
{
	if (this->stitchEnd)this->stitchEnd->removeDependency(this);
	this->stitchEnd = stitch;
	stitch->addDependency(this);
	setNeedRefresh();
}

void OE_linkstitch::setStepPts( std::vector<vector_2d> pts)
{
	stepPts.clear();
	stepPts = pts;
}

bool OE_linkstitch::check(){return (stitchStart&&stitchEnd&&stitchStart->getNpts()&&stitchEnd->getNpts());}

void OE_linkstitch::refresh(){}
void OE_linkstitch::refresh(float dpi)
{
	stitchStart->refresh();
	stitchEnd->refresh();
	
	if (check()&&needRefresh)
	{
			
		stitchStart->refresh();
		stitchEnd->refresh();
		thread = stitchStart->getThread();
		vector_2d tmpPt;
		
		std::vector<vector_2d> 	outPts;
		stitchStart->getPoint(stitchStart->getNpts()-1, &tmpPt);
		outPts.push_back(tmpPt);
		
		outPts.insert(outPts.end(), stepPts.begin(), stepPts.end());
		
		stitchEnd->getPoint(0, &tmpPt);
		outPts.push_back(tmpPt);
		
		
		pts.clear();
		
		if(maxLen>0)
			pts = stitchMaxLen(outPts, maxLen);
		else
			pts = outPts;
	}
	needRefresh = false;
}

void OE_linkstitch::delDependency(OE_base * object)
{
	if (stitchStart == object) stitchStart = 0;
	if (stitchEnd == object) stitchEnd = 0;
}
